/*
	This file contains inline wrappers to make plugin code a little easier to read
	Note this assumes that there is an instance variable defined as:
		const ArrangeCallbackTbl* calls; // callback table
	like the one maintained in the base class 'Plugin'. Plugin.h includes this
	file as part of the 'Plugin' class definition, so you won't need to unless
	you're not decended from the 'Plugin' class.

	Additonally, if the comiler you're using allows you to turn off inlining
	(like Codewarrior does, for instance) you can easily profile the time
	the plugin is spending inside Arrange, without incurring the performance
	cost of using wrapper functions in shipping code.

	Paul Chambers, March 12th, 1995.
*/

/* The callback functions available for manipulating modules themselves. */
	
	inline 	ModuleID	FindModuleName (const char* name, uInteger minVers)
		{ return calls->modMgr->FindModuleName (name, minVers); };
	inline 	ModuleID	FindModuleID (uInteger id, uInteger minVers)
		{ return calls->modMgr->FindModuleID ( id, minVers); };
	inline 	const ModuleInfo*	GetModInfo (pModuleID module)
		{ return calls->modMgr->GetModInfo ( module); };
	inline 	OSErr		OpenModFile (const FSSpec *file, pShort fileRefnum)
		{ return calls->modMgr->OpenModFile ( file, fileRefnum); };
	
	inline 	ModuleEntryPoint  GetModuleEntry (pModuleID module, uInteger entryID)
		{ return calls->modMgr->GetModuleEntry ( module, entryID); };
	inline	ProcPtr		ModuleCallPrelude ( const ModuleEntryPoint *entry,
														  ModuleParamBlock *pb)
		{ return calls->modMgr->ModuleCallPrelude ( entry, pb); };
	inline	void		ModuleCallEpilogue ( const ModuleEntryPoint *entry,
											 const ModuleParamBlock *pb)
		{ calls->modMgr->ModuleCallEpilogue ( entry, pb); };

/* The callback functions available for memory allocation and management. */
	
	inline	void* AllocMem ( uInteger size, AllocMode mode )
		{ return calls->mem->AllocMem ( size, mode ); };
	inline	void DeallocMem ( void* block, AllocMode mode )
		{ calls->mem->DeallocMem ( block, mode ); };
	inline	uInteger MemAvailable ()
		{ return calls->mem->MemAvailable (); };

/* various utility callback functions. */

	inline	const char* GetRString (pShort rsrcID, pShort stringIndex)
		{ return calls->util->GetRString ( rsrcID, stringIndex); };
	inline	void		LowerString (char* string, Integer length)
		{ calls->util->LowerString ( string, length); };

/* The callback functions available for debugging purposes.
	Note: only available if the application has been compiled for module debugging. */
	inline	void PrintToLog (const char* message)
		{ calls->dbg->PrintToLog (message); };

/* The callback functions for manipulating fields. */

	// get field's value
	inline 	Integer		GetFieldTextLen (arNoteID note, arFieldID field)
		{ return calls->data->GetFieldTextLen (note, field); };
	inline 	Integer		GetFieldText ( arNoteID note, arFieldID field,
										Integer bufLen, void* buffer)
		{ return calls->data->GetFieldText ( note, field, bufLen, buffer); };
	inline 	arDate		GetFieldDate (arNoteID note, arFieldID field)
		{ return calls->data->GetFieldDate (note, field); };
	inline 	arFloat		GetFieldNumber (arNoteID note, arFieldID field)
		{ return calls->data->GetFieldNumber (note, field); };
	
	inline 	Integer		GetFieldListLen (arNoteID note, arFieldID field)
		{ return calls->data->GetFieldListLen (note, field); };
	inline	arNoteID	GetFieldListEntry ( arNoteID note, arFieldID field, Integer index )
		{ return calls->data->GetFieldListEntry ( note, field, index); };
	inline 	arListID	GetFieldList (arNoteID note, arFieldID field)
		{ return calls->data->GetFieldList (note, field); };
	
	inline 	PicHandle	GetFieldPict (arNoteID note, arFieldID field)
		{ return calls->data->GetFieldPict (note, field); };
	inline 	Handle		GetFieldAlias (arNoteID note, arFieldID field)
		{ return calls->data->GetFieldAlias (note, field); };
	
	// set field's value
	inline	void SetFieldText ( arNoteID note, arFieldID field,
								const char* text, Integer textLen, arSetFieldFlags flags)
		{ calls->data->SetFieldText ( note, field, text, textLen, flags); };
	inline	void SetFieldDate ( arNoteID note, arFieldID field,
								arDate date, arSetFieldFlags flags)
		{ calls->data->SetFieldDate ( note, field, date, flags); };
	inline	void SetFieldNumber ( arNoteID note, arFieldID field,
								const arFloat *number, arSetFieldFlags flags)
		{ calls->data->SetFieldNumber ( note, field, number, flags); };
	inline	void SetFieldList ( arNoteID note, arFieldID field,
								arListID list, arSetFieldFlags flags)
		{ calls->data->SetFieldList ( note, field, list, flags); };
	inline	void AddListFieldEntry ( arNoteID note, arFieldID field,
									 Integer index, arNoteID childNote,
									 arSetFieldFlags flags)
		{ calls->data->AddListFieldEntry ( note, field, index, childNote, flags); };
	inline	void DeleteListFieldEntry ( arNoteID note, arFieldID field,
										arNoteID childNote, arSetFieldFlags flags)
		{ calls->data->DeleteListFieldEntry ( note, field, childNote, flags); };
	inline	void MoveListFieldEntry ( arNoteID note, arFieldID field,
										arNoteID childNote, Integer index,
										arSetFieldFlags flags)
		{ calls->data->MoveListFieldEntry ( note, field, childNote, index, flags); };
	inline	void SetFieldPict ( arNoteID note, arFieldID field,
										PicHandle pict, arSetFieldFlags flags)
		{ calls->data->SetFieldPict ( note, field, pict, flags); };
	inline	void SetFieldAlias ( arNoteID note, arFieldID field,
										Handle alias, arSetFieldFlags flags)
		{ calls->data->SetFieldAlias ( note, field, alias, flags); };

	// Functions added in version 2 of DataCalls record
	inline 	void ReplaceFieldText ( arNoteID note, arFieldID field,
										Integer repStart, Integer repEnd,
										const char* insText, arSetFieldFlags flags)
		{ calls->data->ReplaceFieldText ( note, field, repStart, repEnd, insText, flags); };
	inline 	Integer GetFieldStyle ( arNoteID note, arFieldID field,
										arFieldStyle* fieldStyle, Integer bufLen,
										arCharStyle* charStyles)
		{ return calls->data->GetFieldStyle ( note, field, fieldStyle, bufLen, charStyles); };
	inline 	void    SetFieldStyle ( arNoteID note, arFieldID field,
										const arFieldStyle* fieldStyle,
										Integer charStyleCount, const arCharStyle* charStyles,
										arSetFieldFlags flags)
		{ calls->data->SetFieldStyle ( note, field, fieldStyle, charStyleCount, charStyles, flags); };


/* The callback functions for creating and destroying notes and managing their fields. */
	
	inline 	arNoteID	 CreateNote ( arTypeID type, pBoolean doGather)
		{ return calls->notes->CreateNote ( type, doGather); };
	inline 	arNoteID	 DuplicateNote ( arNoteID note, pBoolean doGather)
		{ return calls->notes->DuplicateNote ( note, doGather); };
	inline 	void		 ConvertNote ( arNoteID note, arTypeID type, pBoolean doGather)
		{ calls->notes->ConvertNote ( note, type, doGather); };
	inline 	void		 DestroyNote ( arNoteID note)
		{ calls->notes->DestroyNote ( note); };
	
	inline 	void		 AddField ( arNoteID note, arFieldID field, arFieldID nextField)
		{ calls->notes->AddField ( note, field, nextField); };
	inline 	void		 RemoveField ( arNoteID note, arFieldID field)
		{ calls->notes->RemoveField ( note, field); };
	inline 	void		 HideField ( arNoteID note, arFieldID field)
		{ calls->notes->HideField ( note, field); };
	inline 	void		 MoveField ( arNoteID note, arFieldID field, arFieldID nextField)
		{ calls->notes->MoveField ( note, field, nextField); };
	
	inline 	Integer	 CountNoteFields ( arNoteID note)
		{ return calls->notes->CountNoteFields ( note); };
	inline 	arFieldID GetNoteField ( arNoteID note, Integer index)
		{ return calls->notes->GetNoteField ( note, index); };
	inline 	Boolean	 NoteHasField ( arNoteID note, arFieldID field, pBoolean onlyIfVisible)
		{ return calls->notes->NoteHasField ( note, field, onlyIfVisible); };
	
	inline 	Boolean	 NoteExists ( arNoteID note)
		{ return calls->notes->NoteExists ( note); };
	inline 	arTypeID	 GetNoteType ( arNoteID note)
		{ return calls->notes->GetNoteType ( note); };
	
	inline 	void		 GetNoteInfo ( arNoteID note, arNoteInfo *info)
		{ calls->notes->GetNoteInfo ( note, info); };
	inline 	void		 SetNoteInfo ( arNoteID note, const arNoteInfo *info)
		{ calls->notes->SetNoteInfo ( note, info); };

/* The callback functions for inspecting and altering the selection. */

	inline arWindowID	GetActiveWindow ()
		{ return calls->sel->GetActiveWindow (); };
	
	inline arSelType GetSelection ( arNoteID *note, arFieldID *field,
										Integer *selStart, Integer *selEnd)
		{ return calls->sel->GetSelection ( note, field, selStart, selEnd); };
	inline arPathID	GetSelPath ( )
		{ return calls->sel->GetSelPath ( ); };
	inline 	void		GetSelEntry ( Integer index, arNoteID *note,
										arFieldID *field, arNoteID *parentNote)
		{ calls->sel->GetSelEntry ( index, note, field, parentNote); };
	
	inline 	void		SelectObject ( arNoteID parent, arFieldID field,
										arNoteID note, pBoolean doDeselect)
		{ calls->sel->SelectObject ( parent, field, note, doDeselect); };
	inline Boolean		ObjectIsSelected( arNoteID parent, arFieldID field, arNoteID note )
		{ return calls->sel->ObjectIsSelected( parent, field, note); };
	inline 	Boolean	 FlushSelection ( pBoolean alsoDrop)
		{ return calls->sel->FlushSelection ( alsoDrop); };
	
	inline 	void		 ShowPath ( arPathID path, pBoolean selectFlag,
										Integer textSelStart, Integer textSelEnd)
		{ calls->sel->ShowPath ( path, selectFlag, textSelStart, textSelEnd); };
	
	inline 	Integer	 GetSelText ( Integer bufLen, void* buffer,
										Integer *selStart, Integer *selEnd)
		{ return calls->sel->GetSelText ( bufLen, buffer, selStart, selEnd); };
	inline 	void		 SetSelText ( const char* text, Integer textLen,
										Integer selStart, Integer selEnd)
		{ calls->sel->SetSelText ( text, textLen, selStart, selEnd); };
	
	// Functions added in version 2 of SelCalls record
	inline 	void	  ReplaceSelText ( const char* text, const char* undoName)
		{ calls->sel->ReplaceSelText ( text, undoName); };

/* The callback functions for searching, sorting, and filtering. */
	
	// Searching
	inline Boolean NextAppearance ( arPathID path, arDirCode direction )
		{ return calls->search->NextAppearance( path, direction ); };
	inline 	Boolean FindText ( arPathID path, arDirCode direction,
								Integer *selStart, Integer *selEnd,
								const char* searchString, arSearchFlags flags)
		{ return calls->search->FindText ( path, direction, selStart, selEnd,
											searchString, flags); };
	
	// Filtering and sorting
	inline 	arListID FilterList ( arListID list, arTypeID matchType, arFilterFlags flags,
									pShort clauseCount, const arFilterClause* clauses)
		{ return calls->search->FilterList ( list, matchType, flags, clauseCount, clauses); };
	inline 	arListID FilterNotes ( arTypeID type, arFilterFlags flags,
									 pShort clauseCount, const arFilterClause* clauses)
		{ return calls->search->FilterNotes ( type, flags, clauseCount, clauses); };
	inline 	arListID SortNotes ( arListID list,
								 pShort clauseCount, const arSortClause* clauses)
		{ return calls->search->SortNotes ( list, clauseCount, clauses); };

/* The callback functions for creating and accessing system objects
	(i.e. topics, note type definitions, and so forth). */
	
	// General calls
	inline 	arNoteID		 GetBuiltInObject ( arBuiltInObject whichObject)
		{ return calls->sysObj->GetBuiltInObject ( whichObject); };
	inline 	arNoteID		 LookupObjectName ( arObjectClass objClass,
												const char* name, pBoolean allowPartial)
		{ return calls->sysObj->LookupObjectName ( objClass, name, allowPartial); };
	
	// Topic/folder/view calls
	inline 	arTopicID	 CreateTopic ( const char* name, arTopicID parent,
										 arTopicID successor, pBoolean makeView)
		{ return calls->sysObj->CreateTopic ( name, parent, successor, makeView); };
	inline 	arTopicID	 CreateFolder ( const char* name, arTopicID successor)
		{ return calls->sysObj->CreateFolder ( name, successor); };
	inline 	void			 UpdateView ( arTopicID view)
		{ calls->sysObj->UpdateView ( view); };
	inline 	void			 GetTopicInfo ( arTopicID topic, arTopicInfo *info)
		{ calls->sysObj->GetTopicInfo ( topic, info); };
	inline 	void			 SetTopicInfo ( arTopicID topic, const arTopicInfo *info)
		{ calls->sysObj->SetTopicInfo ( topic, info); };
	
	// Note type calls
	inline 	arTypeID	CreateNoteType ( const char* name, pBoolean userVisible)
		{ return calls->sysObj->CreateNoteType ( name, userVisible); };
	inline	Integer		CountTypeInstances ( arTypeID type )
		{ return calls->sysObj->CountTypeInstances( type ); };
	inline 	arListID	GetTypeInstances ( arTypeID type)
		{ return calls->sysObj->GetTypeInstances ( type); };
	inline 	void		GetTypeInfo ( arTypeID type, arTypeInfo *info)
		{ calls->sysObj->GetTypeInfo ( type, info); };
	inline 	void		SetTypeInfo ( arTypeID type, const arTypeInfo *info)
		{ calls->sysObj->SetTypeInfo ( type, info); };
	
	// Field calls
	inline 	arFieldID	 CreateField ( const char* name, arFieldType dataType,
										pBoolean userVisible)
		{ return calls->sysObj->CreateField ( name, dataType, userVisible); };
	inline 	void			 GetFieldInfo ( arFieldID field, arFieldInfo *info)
		{ calls->sysObj->GetFieldInfo ( field, info); };
	inline 	void			 SetFieldInfo ( arFieldID field, const arFieldInfo *info)
		{ calls->sysObj->SetFieldInfo ( field, info); };
	
	// Window calls
	inline 	arWindowID	 CreateWindow ( const char* name, const Rect *bounds)
		{ return calls->sysObj->CreateWindow ( name, bounds); };
	inline 	arTopicID	 GetCurrentTopic ( arWindowID window, pBoolean returnRoot)
		{ return calls->sysObj->GetCurrentTopic ( window, returnRoot); };
	inline Boolean		SetCurrentTopic	( arWindowID window, arTopicID topic,
											pBoolean useLastView )
		{ return calls->sysObj->SetCurrentTopic ( window, topic, useLastView); };
	inline 	Boolean		 SelectWindow ( arWindowID window)
		{ return calls->sysObj->SelectWindow ( window); };
	inline 	void			 GetWindInfo ( arWindowID window, arWindInfo *info)
		{ calls->sysObj->GetWindInfo ( window, info); };
	inline 	void			 SetWindInfo ( arWindowID window, const arWindInfo *info)
		{ calls->sysObj->SetWindInfo ( window, info); };

/* The callback functions for manipulating arListIDs and arPathIDs. */
	
	// List access
	inline 	Integer	 GetListLen ( arListID list)
		{ return calls->list->GetListLen ( list); };
	inline 	arNoteID	 GetListEntry ( arListID list, Integer index)
		{ return calls->list->GetListEntry ( list, index); };
	inline 	Integer	 GetListEntries ( arListID list, Integer index,
										arNoteID* buffer, Integer bufSize)
		{ return calls->list->GetListEntries ( list, index, buffer, bufSize); };
	inline 	Integer	 SearchList ( arListID list, arNoteID target)
		{ return calls->list->SearchList ( list, target); };
	
	// List editing
	inline 	void		 SetListEntry ( arListID *list, Integer index, arNoteID entry)
		{ calls->list->SetListEntry ( list, index, entry); };
	inline 	void		 SetListEntries ( arListID *list, Integer index,
											const arNoteID* entries, Integer count)
		{ calls->list->SetListEntries ( list, index, entries, count); };
	inline 	void		 InsertListEntry ( arListID *list, Integer index, arNoteID entry)
		{ calls->list->InsertListEntry ( list, index, entry); };
	inline 	void		 InsertListEntries ( arListID *list, Integer index, Integer count)
		{ calls->list->InsertListEntries ( list, index, count); };
	inline 	void		 DeleteListEntries ( arListID *list, Integer index, Integer count)
		{ calls->list->DeleteListEntries ( list, index, count); };
	inline 	void		 ResizeList ( arListID *list, Integer newSize)
		{ calls->list->ResizeList ( list, newSize); };
	
	// Miscellaneous list operations
	inline 	arListID CreateList ( Integer length)
		{ return calls->list->CreateList ( length); };
	inline 	arListID DupList ( arListID list)
		{ return calls->list->DupList ( list); };
	inline 	void			 DisposeList ( arListID list)
		{ calls->list->DisposeList (list); };
	
	// Path access
	inline 	arPathType GetPathInfo ( arPathID path, arNoteID *root)
		{ return calls->list->GetPathInfo ( path, root); };
	inline 	Integer	 GetPathLen ( arPathID path)
		{ return calls->list->GetPathLen ( path); };
	inline 	arNoteID	 GetPathEntry ( arPathID path, Integer index)
		{ return calls->list->GetPathEntry ( path, index); };
	
	// Path manipulation
	inline 	Boolean	 PushPath ( arPathID path, arNoteID newEntry)
		{ return calls->list->PushPath ( path, newEntry); };
	inline 	void		 PopPath ( arPathID path)
		{ calls->list->PopPath ( path); };
	inline 	Boolean	 WalkPath ( arPathID path, arDirCode direction, arWalkCache* cache)
		{ return calls->list->WalkPath ( path, direction, cache); };
	inline void		DisposeWalkCache ( arWalkCache cache )
		{ calls->list->DisposeWalkCache (cache); };
	
	// Miscellaneous path operations
	inline 	arPathID	PathToTopic ( arTopicID topic, pBoolean atEnd)
		{ return calls->list->PathToTopic ( topic, atEnd); };
	inline 	Boolean		EqualPaths ( arPathID path1, arPathID path2)
		{ return calls->list->EqualPaths ( path1, path2); };
	inline 	arPathID	DupPath ( arPathID path)
		{ return calls->list->DupPath ( path); };
	inline 	void		DisposePath ( arPathID path)
		{ calls->list->DisposePath ( path); };

/* The callback functions for manipulating the display state
	(scrolling, opening and closing notes, etc.). */
	
	inline arNoteState	GetNoteState ( arNoteID note, arNoteID parentNote )
		{ return calls->view->GetNoteState ( note, parentNote ); };
	inline 	void		SetNoteState ( arNoteID note, arNoteID parentNote,
										arNoteState newState)
		{ calls->view->SetNoteState ( note, parentNote, newState); };

/* The callback functions for document-level operations
	(e.g. opening, closing and saving documents). */
	
	inline arDocumentPtr	GetCurrentDoc ()
		{ return	calls->doc->GetCurrentDoc (); };
	inline 	void			SetCurrentDoc (arDocumentPtr newDoc)
		{ calls->doc->SetCurrentDoc (newDoc); };
	
	inline Integer			CountOpenDocs ()
		{ return calls->doc->CountOpenDocs (); };
	inline 	arDocumentPtr GetIndexedDoc (Integer index)
		{ return calls->doc->GetIndexedDoc (index); };
	inline arDocumentPtr	GetPrefsDoc ()
		{ return calls->doc->GetPrefsDoc (); };
	
	inline 	void			 GetDocInfo (arDocInfo *info)
		{ calls->doc->GetDocInfo (info); };
	
	inline 	arDocumentPtr NewDoc ( const FSSpec* stationeryFile, const char* title )
		{ return calls->doc->NewDoc ( stationeryFile, title ); };
	inline 	arDocumentPtr Open ( const FSSpec* srcFile, pBoolean readOnly)
		{ return calls->doc->Open ( srcFile, readOnly); };
	inline	void		  Close	()
		{ calls->doc->Close (); };
	
	inline	arFileOpResult	Save ()
		{ return calls->doc->Save (); };
	inline 	arFileOpResult	SaveAs (const FSSpec* targetFile)
		{ return calls->doc->SaveAs (targetFile); };
	inline 	arFileOpResult	Revert ()
		{ return calls->doc->Revert (); };
	
	inline 	Integer	GetGlobalData  ( const char* name, Integer bufLen, void* buffer)
		{ return calls->doc->GetGlobalData ( name, bufLen, buffer ); };
	inline	void	SetGlobalData  ( const char* name, const void* data, Integer dataLen,
									 pBoolean isText)
		{ calls->doc->SetGlobalData  ( name, data, dataLen, isText); };
	inline	Boolean	BringCurDocToFront ()
		{ return calls->doc->BringCurDocToFront (); };
	inline	void	SetChangedFlag ()
		{ calls->doc->SetChangedFlag (); };
	inline 	void	SetupUndo ( const char* operationName, pBoolean setChangedFlag)
		{ calls->doc->SetupUndo ( operationName, setChangedFlag); };
	
	inline 	void	OpenHiddenDoc ( const FSSpec* srcFile, pBoolean readOnly,
									arDocumentPtr* doc, Boolean* closeWhenFinished)
		{ calls->doc->OpenHiddenDoc ( srcFile, readOnly, doc, closeWhenFinished); };

/* The callback functions for accessing and extending
	Arrange's import & export capabilities. */

	inline 	Boolean ImportTAIL (const FSSpec* srcFile,    arTopicID destTopic)
		{ return calls->io->ImportTAIL (srcFile, destTopic); };
	inline 	Boolean ExportTAIL (const FSSpec* targetFile, arTopicID srcTopic )
		{ return calls->io->ExportTAIL (targetFile, srcTopic ); };

/* The callback functions for customizing the Arrange user interface. */
	
	inline 	void AddMenu ( const char* menuName, pShort menuCode, arMenuID parentCode)
		{ calls->ui->AddMenu ( menuName, menuCode, parentCode); };
	inline 	void AddMenuItem ( pShort menuCode, const char* itemName, pShort cmdChar,
								Integer commandCode, Integer itemRefcon)
		{ calls->ui->AddMenuItem ( menuCode, itemName, cmdChar, commandCode, itemRefcon); };
	inline 	void DeleteMenuItem ( pShort menuCode, Integer commandCode, Integer itemRefcon)
		{ calls->ui->DeleteMenuItem ( menuCode, commandCode, itemRefcon); };
	inline 	void SetMenuItem ( pShort menuCode, Integer commandCode, Integer itemRefcon,
								const char* itemName, pBoolean itemEnabled,
								pShort markChar, pShort itemStyle)
		{ calls->ui->SetMenuItem ( menuCode, commandCode, itemRefcon, itemName,
									itemEnabled, markChar, itemStyle); };
	
	inline 	void SetClickHook ( ClickHook hook, uInteger refcon, pBoolean addHook)
		{ calls->ui->SetClickHook ( hook, refcon, addHook); };
	inline 	void SetMenuHook ( MenuHook hook, uInteger refcon, pBoolean addHook,
								Integer commandCode)
		{ calls->ui->SetMenuHook ( hook, refcon, addHook, commandCode); };
	inline 	void SetKeyHook ( KeyHook hook, uInteger refcon, pBoolean addHook,
								pShort charFilter, pShort keyFilter, pShort modFilter)
		{ calls->ui->SetKeyHook ( hook, refcon, addHook, charFilter, keyFilter, modFilter); };
	inline 	void SetFieldHook ( FieldHook hook, uInteger refcon, pBoolean addHook,
								arFieldID field)
		{ calls->ui->SetFieldHook ( hook, refcon, addHook, field); };
	inline 	void SetTopicHook ( TopicHook hook, uInteger refcon, pBoolean addHook)
		{ calls->ui->SetTopicHook ( hook, refcon, addHook); };
	inline 	void SetTickHook ( TickHook  hook, uInteger refcon, pBoolean addHook)
		{ calls->ui->SetTickHook ( hook, refcon, addHook); };
	inline 	void SetFileHook ( FileHook  hook, uInteger refcon, pBoolean addHook)
		{ calls->ui->SetFileHook ( hook, refcon, addHook); };
	inline 	void SetQuitHook ( QuitHook  hook, uInteger refcon, pBoolean addHook)
		{ calls->ui->SetQuitHook ( hook, refcon, addHook); };
	
	inline 	void SetATMHook ( AboutToMenuHook hook, uInteger refcon, pBoolean addHook)
		{ calls->ui->SetATMHook ( hook, refcon, addHook); };
	
	inline 	void RegisterImportFormat ( const char* name, Integer formatID,
										OSType* fileTypes, Integer fileTypeCount,
										ImportHook hook, uInteger refcon)
		{ calls->ui->RegisterImportFormat ( name, formatID, fileTypes, fileTypeCount,
											hook, refcon); };
	
	inline 	void RegisterExportFormat ( const char* name, Integer formatID,
										arEFType type, ExportHook hook,
										uInteger refcon, OSType fileType,
										OSType fileCreator)
		{ calls->ui->RegisterExportFormat ( name, formatID, type, hook, refcon,
											fileType, fileCreator); };
	
	inline	void RegisterFieldProc( const char* name, Integer id, arFPType type,
									Integer fieldTypes, FieldHook proc, uInteger refcon)
		{ calls->ui->RegisterFieldProc ( name, id, type, fieldTypes, proc, refcon ); };
	inline 	Boolean TestFieldProc ( arFieldID field, arFPType type, Integer procID)
		{ return calls->ui->TestFieldProc ( field, type, procID); };

/* The callback functions for displaying dialogs and progress indicators. */

	inline 	void	 StopAlert ( const char* message)
		{ calls->dlg->StopAlert ( message); };
	inline 	Boolean OKCancelAlert ( const char* message)
		{ return calls->dlg->OKCancelAlert ( message); };
	inline 	Boolean PromptForString ( const char* prompt, char* string,
										pShort maxLen, pBoolean numbersOnly)
		{ return calls->dlg->PromptForString ( prompt, string, maxLen, numbersOnly); };
	
	inline 	Boolean PromptForField ( const char* prompt, arFieldID* field,
									 const arFieldType* allowedTypes,
									 arChoiceDialogFlags flags)
		{ return calls->dlg->PromptForField ( prompt, field, allowedTypes, flags); };
	inline 	Boolean PromptForType ( const char* prompt, arTypeID* type,
									arChoiceDialogFlags flags)
		{ return calls->dlg->PromptForType ( prompt, type, flags); };
	
	inline 	void	DisplayNotify (const char* message, arNotifyFlags flags)
		{ calls->dlg->DisplayNotify (message, flags); };
	inline void		ClearNotify ()
		{ calls->dlg->ClearNotify (); };
	inline 	void	SetNotifyText (const char* newMessage)
		{ calls->dlg->SetNotifyText (newMessage); };
	
	inline 	Boolean TickNotify (Integer amountDone, Integer totalAmount)
		{ return calls->dlg->TickNotify (amountDone, totalAmount); };
	
	inline	void	BackgroundTick ()
		{ calls->dlg->BackgroundTick (); };

/* End of the inline function definitions for Arrange's callbacks */